/* libtcod
 * Copyright © 2008-2018 Jice and the libtcod contributers.
 * All rights reserved.
 *
 * libtcod 'The Doryen library' is a cross-platform C/C++ library for roguelike
 * developers.
 * Its source code is available from:
 * https://github.com/libtcod/libtcod
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of copyright holder nor the names of its contributors may not
 *       be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef LIBTCOD_ENGINE_DISPLAY_H_
#define LIBTCOD_ENGINE_DISPLAY_H_

#ifdef __cplusplus
#include <string>
#endif // __cplusplus

#include "../tileset/tileset.h"
#include "../console_types.h"

#ifdef __cplusplus
namespace tcod {
namespace engine {
using tcod::tileset::Tileset;
class Display {
 public:
  virtual ~Display() = default;
  virtual void set_tileset(std::shared_ptr<Tileset> tileset) = 0;
  virtual void set_title(const std::string title) = 0;
  virtual std::string get_title() = 0;
  /**
   *  Set the fullscreen status if the display supports it.
   *
   *  When it is suppered this can be used to switch between a window and
   *  borderless fullscreen window.
   */
  virtual void set_fullscreen(bool fullscreen) = 0;
  /**
   *  Return true if the display is in fullscreen mode.
   *
   *  Returns a negative number if the display does not support fullscreen.
   */
  virtual int get_fullscreen() = 0;
  virtual void present(const TCOD_Console*) = 0;
};
/**
 *  Incomplete interface for subclasses which don't need an SDL2 window.
 */
class TerminalDisplay: public Display {
 public:
  virtual void set_tileset(std::shared_ptr<Tileset> tileset) override
  {}
  virtual void set_title(const std::string title) override
  {}
  virtual std::string get_title() override
  {
    return {};
  }
  /**
   *  Terminals do not support fullscreen modes.
   */
  virtual void set_fullscreen(bool fullscreen)
  {}
  /**
   *  Return a negative error code.
   */
  virtual int get_fullscreen()
  {
    return -1;
  }
};
} // namespace sdl2
} // namespace tcod
#endif // __cplusplus
#endif // LIBTCOD_ENGINE_DISPLAY_H_


